<?php
/**
 * Thor_ItemsContainer Class.
 * Provides the functions to deal with Thor_Items.
 *
 * $Horde: thor/lib/Item.php,v 1.18 2004/12/07 16:09:17 chuck Exp $
 *
 * @author  Carlos Pedrinaci <cpedrinaci@yahoo.es>
 * @package Thor
 */
class Thor_ItemsContainer {

    // DataTree instance holding all the project's items
    var $_items;
    var $_params;

    /**
     * Constructor
     */
    function Thor_ItemsContainer()
    {
        global $conf;
        if (empty($conf['datatree']['driver'])) {
            Horde::fatal(_("You must configure a Horde DataTree backend to use Thor."), __FILE__, __LINE__);
        }
        $driver = $conf['datatree']['driver'];
        $cparams = array_merge(Horde::getDriverConfig('datatree', $driver), array('group' => 'thor.items'));

        $this->_items = &DataTree::singleton($driver, $cparams);
    }

    function &singleton()
    {
        static $items;

        if (!isset($items)) {
            $items = new Thor_ItemsContainer();
        }

        return $items;
    }

    /**
     * Items factory
     *
     * @param string $item_path
     * @param string $kind The kind of Item we want to create
     * @return mixed PEAR_Error if this kind of Item is unknown. The Item if all goes ok.
     */
    function &newItem($item_path, $kind)
    {
        require_once dirname(__FILE__) . '/Item/' . $kind . '.php';
        $class = 'DataTreeObject_' . $kind;
        if (class_exists($class)) {
            $item = &new $class($item_path);
            return $item;
        } else {
            return PEAR::raiseError(sprintf(_("'%s' not found."), $class));
        }
    }

    function &getItem($item_id)
    {
        $item = &$this->_items->getObjectById($item_id, null);
        if (is_a($item, 'PEAR_Error')) {
            return $item;
        }
        return $item;
    }

    function getItemId($item_name)
    {
        return $this->_items->getId($item_name);
    }

    function getItemParent($item)
    {
        return $this->_items->getParent($item);
    }

    /**
     * This should be modified to use the method removeById which is
     * not yet implemented in the DataTree driver
     *
     * @param integer $itemID The item identifier
     * @return mixed PEAR_Error if something went wrong. The item id if all went ok.
     */
    function deleteItem($item_id)
    {
        $item = &$this->getItem($item_id);
        if (is_a($item, 'PEAR_Error')) {
            return $item;
        }
        $item->_destroy();
        return $this->_items->remove($item->getName());
    }

    /**
     * Function that updates the Last Modified Date of the Item.
     * It does also update any of its parents
     *
     * @param integer $itemID The item identifier
     * @return mixed PEAR_Error if something went wrong. True if all went ok.
     */
    function updateModifiedDate($itemID)
    {
        global $thor;

        $theItem = &$this->getItem($itemID);
        $now = time();
        $theItem->set('item_last_modified', $now);
        $result = $this->_items->updateData($theItem);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        //Update the upper level items
        if (!$this->isTopLevelItem($itemID)) {
            $item_parent = $this->getItemParent($theItem);
            $result = $this->updateModifiedDate($item_parent);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }
        return true;
    }

    /**
     * Updates or Adds the Item to the storage backend.
     * Updates the Project's Last Modified date
     * When updating it updates the Last Modified date.
     * @param mixed $info The Item array or an DataTreeObject_Item instance
     */
    function saveItem(&$info)
    {
        global $thor;

        if (is_a($info, 'DataTreeObject_Item')) {
            //Existing Item: Update it
            $info->set('item_last_modified', time());
            $result = $this->_items->updateData($info);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
            // Update Item parents and Projects last modified attribute
            if (!$this->isTopLevelItem($info)) {
                $this->updateModifiedDate($this->getItemParent($info));
            }
            $thor->_projectsContainer->updateModifiedDate($info->get('project_id'));
            return $info->get('item_id');
        }

        if (empty($info['item_id'])) {
            // Its a new Item
            $item_name = $info['item_name'];
            $item_kind = $info['item_kind'];
            $item_parent_id = $info['item_parent_id'];
            if ($item_parent_id != DATATREE_ROOT) {
                $item_path = $this->_items->getName($item_parent_id);
            } else {
                $item_path = $thor->_projectsContainer->_projects->getName($info['project_id']);
            }
            $item_path .= ':' . DataTree::encodeName($item_name);
            $item = &$this->newItem($item_path, $item_kind);
            $item->data['project_id'] = $info['project_id'];
        } else {
            $item = &$this->getItem($info['item_id']);
        }
        $item->fromArray($info);
        if (empty($info['item_id'])) {
            // Its a new Item
            $result = $this->_items->add($item);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        } else {
            //Existing Item: Update it
            $item->set('item_last_modified', time());
            $result = $this->_items->updateData($item);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
            $result = $info['item_id'];
        }
        // Update Item parents and Projects last modified attribute
        if (!$this->isTopLevelItem($item)) {
            $this->updateModifiedDate($this->getItemParent($item));
        }
        $thor->_projectsContainer->updateModifiedDate($item->get('project_id'));
        return $result;
    }

    /**
     * Function that retrieves the list of Items
     * @param string $startleaf The node from which to start getting the list.
     * @param integer $maxdepth The depth of the tree
     * @param boolean $getObjects True: get also the Items instances
     * @return mixed All ok hash id => name or name => Item_instance if $getObjects is true
     */
    function &getItems($startleaf, $maxdepth = -1, $no_root = false, $getObjects = false)
    {
        $itemsList = $this->_items->get(DATATREE_FORMAT_FLAT, $startleaf, true,  _("-- Top Level --"), $maxdepth, true);
        if (is_a($itemsList, 'PEAR_Error')) {
            Horde::logMessage($itemsList, __FILE__, __LINE__, PEAR_LOG_ERR);
            return array();
        }

        if ($no_root) {
            $itemId = $this->getItemId($startleaf);
            unset($itemsList[$itemId]);
        }

        if ($getObjects) {
            // Get the Item Objects
            $keys = array_keys($itemsList);
            $itemsList = &$this->_items->getObjects($keys, null);
        }

        // TODO:Sort the list
        return $itemsList;
    }

    /**
     * Given an item object or identifier it determines whether it is top level or not
     *
     * @param mixed  $item The identifier or a DataTreeObject_Item
     * @return mixed True if the item is Top Level.
     *                        PEAR_Error if there was any error getting the item
     */
    function isTopLevelItem($item)
    {
        global $thor;

        if (!is_a($item, 'DataTreeObject_Item')) {
            $item = &$this->getItem($item);
            if (is_a($item, 'PEAR_Error')) {
                Horde::logMessage($item, __FILE__, __LINE__, PEAR_LOG_ERR);
                return $item;
            }
        }

        $parent_id = $this->_items->getParent($item);
        $parent_name = $this->_items->getName($parent_id);

        $project_id = $item->get('project_id');
        $theProject = &$thor->_projectsContainer->getProject($project_id);
        $project_name = $theProject->getName();
        if ($project_name == $parent_name) {
            return true;
        }
        return false;
    }

}

/**
 * DataTreeObject_Item Class.
 * Every Item in Thor is an extension of that class which defines common
 * fields to every object as well as methods all should implement
 *
 * $Horde: thor/lib/Item.php,v 1.18 2004/12/07 16:09:17 chuck Exp $
 *
 * @author  Carlos Pedrinaci <cpedrinaci@yahoo.es>
 * @version $Revision: 1.18 $
 * @package Thor
 */
class DataTreeObject_Item extends DataTreeObject {

    /**
     * Variable required for Dynamic Binding of Items
     */
    var $_objectType = 'thor/Item';

    function DataTreeObject_Item($name)
    {
        parent::DataTreeObject($name);
        $this->set('item_creator_id', Auth::getAuth());
        $this->set('item_creation_date', time());
        $this->set('item_status', THOR_STATUS_REQUEST);
        $this->set('item_modifiable', 1);
    }

    function _toAttributes()
    {
        $attributes[] = array('name'  => 'DataTree',
                              'key'   => 'objectType',
                              'value' => $this->_objectType);
        $attributes[] = array('name'  => 'item',
                              'key'   => 'project_id',
                              'value' => $this->data['project_id']);
        $attributes[] = array('name'  => 'item',
                              'key'   => 'creator_id',
                              'value' => $this->data['item_creator_id']);
        $attributes[] = array('name'  => 'item',
                              'key'   => 'creation_date',
                              'value' => $this->data['item_creation_date']);
        $attributes[] = array('name'  => 'item',
                              'key'   => 'last_modified',
                              'value' => $this->data['item_last_modified']);
        $attributes[] = array('name'  => 'item',
                              'key'   => 'description',
                              'value' => $this->data['item_description']);
        $attributes[] = array('name'  => 'item',
                              'key'   => 'status',
                              'value' => $this->data['item_status']);
        if (!empty($this->data['item_write_protected'])) {
            $attributes[] = array('name'  => 'item',
                      'key'   => 'write_protected',
                      'value' => 1);
        }
        return $attributes;
    }

    function _fromAttributes($attributes)
    {
        /* Initialise the data array. */
        $this->data = array();

        foreach ($attributes as $attr) {
            if ($attr['key'] == 'project_id') {
                $this->data['project_id'] = $attr['value'];
            } else {
                $this->data[$attr['name'] . '_' . $attr['key']] = $attr['value'];
            }
        }
    }

    function toArray()
    {
        global $thor;

        $array['item_id'] = $thor->_itemsContainer->getItemId($this->getName());
        $array['project_id'] = $this->data['project_id'];
        $array['item_parent_id'] = $thor->_itemsContainer->getItemParent($this);
        $array['item_creator_id'] = $this->data['item_creator_id'];
        $array['item_name'] = $this->getShortName();
        $array['item_creation_date'] = $this->data['item_creation_date'];
        $array['item_last_modified'] = $this->data['item_last_modified'];
        $array['item_description'] = $this->data['item_description'];
        $array['item_status'] = $this->data['item_status'];
        $array['item_kind'] =  substr(strrchr($this->_objectType, '/'), 1);
        $array['item_write_protected'] = empty($this->data['item_write_protected']) ? 0 : 1;
        return $array;
    }

    function fromArray(&$info)
    {
        $this->data['item_last_modified'] = time();
        $this->data['item_description'] = $info['item_description'];
    }

    /**
     * Show Information about the Item
     * Should be implemented by any subclass
     */
    function showInformation()
    {
        echo sprintf('The class %s must implement the method showInformation', get_class($this));
    }

    /**
     * Download the item
     * Should be implemented by any subclass
     *@return mixed True if data could be sent. PEAR_Error otherwise.
     *
     */
    function download()
    {
        return raise::PEAR_Error(sprintf('The class %s must implement the method download', get_class($this)));
    }

    /**
     * Initialization actions for the Item
     * Everytime an Item is created this function is executed
     */
    function initialize()
    {
        global $thor;

        $this->set('item_status', THOR_STATUS_ACTIVE);
        return $thor->_itemsContainer->saveItem($this);
    }

    /**
     * Write protect the Item.
     * This can be used by Project administrators to write protect items
     *
     *@param boolean $value True -> Write protect it.
     *@return mixed PEAR_Error is something went wrong. Id of the item otherwise
     */
    function setWriteProtected($value)
    {
        global $thor;

        $this->data['item_write_protected'] = $value ? 1 : 0;
        $result = $thor->_itemsContainer->saveItem($this);
        return $result;
    }

     /**
     * The item is being removed
     * This function is executed everytime an item is deleted
     */
    function _destroy()
    {
        global $thor;

         //For each sub Item -> destroy it
        $itemsList = &$thor->_itemsContainer->getItems($this->getName(), 1, true, false);
        $itemIds = array_keys($itemsList);
        foreach ($itemIds as $id) {
            $thor->_itemsContainer->deleteItem($id);
        }
    }

}

/**
 * ItemForm Class
 *
 * $Horde: thor/lib/Item.php,v 1.18 2004/12/07 16:09:17 chuck Exp $
 *
 * @author  Carlos Pedrinaci <cpedrinaci@yahoo.es>
 * @version $Revision: 1.18 $
 * @package Thor
 */
class ItemForm extends Horde_Form {

    var $_useFormToken = false;

    function ItemForm(&$vars)
    {
        global $thor;

        $itemID = $vars->get('item_id');
        $projectID = $vars->get('project_id');
        $parentID = $vars->get('item_parent_id');
        if (!isset($parentID)) {
            $vars->set('item_parent_id', DATATREE_ROOT);
            $parentID = DATATREE_ROOT;
        }

        $title = empty($itemID) ? _("Create Item - Step 1") : _("Edit Item");

        parent::Horde_Form($vars, $title);

        $this->addHidden('', 'item_id', 'int', false);
        $this->addHidden('', 'project_id', 'int', false);
        $this->addHidden('', 'item_parent_id', 'text', false);
        $this->addVariable(_("Item Name"), 'item_name', 'text', true);

        if ($parentID != DATATREE_ROOT) {
            $parentItem = &$thor->_itemsContainer->getItem($parentID);
            $parentData = $parentItem->toArray();
            $kinds_list = $thor->getSubItems($parentData['item_kind'], THOR_ITEMS_AVAILABLE);
            $data = array($parentID => $parentData['item_name']);
        } else {
            $project = &$thor->_projectsContainer->getProject($projectID);
            $projectData = $project->toArray();
            $kinds_list = $thor->getItems($projectData['project_kind'], THOR_ITEMS_AVAILABLE);
            $data = array(DATATREE_ROOT => $projectData['project_name']);
        }
        // Don't allow to change the Item kind when editting
        $this->addVariable(_("Item Kind"), 'item_kind', 'enum', true, empty($itemID) ? false : true, null, array($kinds_list));
        $this->addVariable(_("Item Description"), 'item_description', 'longtext', true, false, null);
    }

}